from isbtchot import model
from isbtchot import view as v
from isbtchot.schemas.args import TypeTime


view = v.ViewPlotText()

def btc_terminal(time_grouping: TypeTime, periods_back: int):
    df = model.btc_hist(time_grouping, periods_back)
    view.candle_stick_terminal(df, "BTC/USD")
    view.plt.yscale("log")

def btc_pi_terminal(time_grouping: TypeTime, periods_back):
    df = model.btc_pi(time_grouping, periods_back)
    view.chart_terminal(df.time, df.pi, "Hotness Index")
    view.add_horizontal_line(1, "red")
    view.add_horizontal_line(0.35, "green")
    for _, row in df.iterrows():
        if row.pi_sell:
            view.add_vertical_line(row.time, "red")
        if row.pi_buy:
            view.add_vertical_line(row.time, "green")
    view.plt.ylim(0.20, 1.1)
    

def dashboard(time_grouping: TypeTime, periods_back: int):
    view.dimensions(0, -4)
    view.subplots(2,1)
    view.select_subplot(1,1)
    btc_terminal(time_grouping, periods_back)
    view.select_subplot(2,1)
    btc_pi_terminal(time_grouping, periods_back)
    view.show()


import requests
import datetime
import pandas as pd
import isbtchot
from isbtchot.schemas.args import TypeTime


CACHE_BTC_PATH = isbtchot.root_path / "cache" / "btc.csv"


def btc_historical_daily() -> pd.DataFrame:
    # Ensure the directory exists
    CACHE_BTC_PATH.parent.mkdir(parents=True, exist_ok=True)

    # Check if the file exists and was modified today
    if (
        CACHE_BTC_PATH.is_file()
        and datetime.datetime.fromtimestamp(CACHE_BTC_PATH.stat().st_mtime).date()
        == datetime.datetime.today().date()
    ):
        df = pd.read_csv(CACHE_BTC_PATH)

    else:
        try:
            data = requests.get(
                "https://min-api.cryptocompare.com/data/v2/histoday?fsym=BTC&tsym=USD&allData=true"
            ).json()["Data"]["Data"]

        except Exception as _:
            # Try without ssl enabled
            data = requests.get(
                "https://min-api.cryptocompare.com/data/v2/histoday?fsym=BTC&tsym=USD&allData=true",
                verify=False
            ).json()["Data"]["Data"]

        df = pd.DataFrame(data)
        df.to_csv(CACHE_BTC_PATH, index=False)

    df["time"] = pd.to_datetime(df["time"], unit="s")
    df.set_index("time", inplace=True)
    return df


def btc_pi(time_grouping: TypeTime, periods_back: int):
    df = btc_historical_daily()[["close"]].rename({"close": "price"}, axis=1)

    df["sma111"] = df["price"].rolling(window=111).mean()
    df["sma350x2"] = df["price"].rolling(window=350).mean() * 2
    df["pi"] = df["sma111"] / df["sma350x2"]

    df = df.drop(["sma111", "sma350x2"], axis=1)
    df = df.dropna()

    # Sell Indicator
    mask_pi_sell = (df["pi"].shift(-1) > 1) & (df["pi"] <= 1)
    df["pi_sell"] = mask_pi_sell

    # Buy indicator
    mask_pi_buy = (df["pi"].shift(-1) < 0.35) & (df["pi"] >= 0.35)
    df["pi_buy"] = mask_pi_buy

    df = df.resample(time_grouping.value).agg(
        {
            "price": "last",
            "pi": "max",
            "pi_sell": "max",
            "pi_buy": "max",
        }
    )

    df = df.reset_index()
    df.time = df.time.dt.strftime("%d/%m/%Y")

    if periods_back:
        df = df.iloc[-periods_back:]

    return df


def btc_hist(time_grouping: TypeTime, periods_back: int):
    df = btc_historical_daily()
    df = df.rename(
        {"open": "Open", "close": "Close", "high": "High", "low": "Low"}, axis=1
    )
    df = df.resample(time_grouping.value).agg(
        {
            "Open": "first",
            "High": "max",
            "Low": "min",
            "Close": "last",
        }
    )
    if periods_back:
        df = df.iloc[-periods_back:]
    return df



from isbtchot import controller
from isbtchot.schemas.args import TypeTime
import argparse

parser = argparse.ArgumentParser(description="BTC Hotness Index", prog="isbtchot")

# Add an optional positional argument named 'year' with default value of 6
parser.add_argument(
    "-periods_back",
    "--p",
    type=int,
    default=85,
    help="Periods back to be processed. Defaults to 50 if not provided.",
)

parser.add_argument(
    "-time",
    "--t",
    type=TypeTime,
    default=TypeTime.WEEK,
    help="Candle stick time to use. Defaults to W (Weekly),"
)


def main():

    # Parse args
    args = parser.parse_args()
    time_grouping: TypeTime = args.t
    periods_back: int = args.p

    controller.dashboard(periods_back=periods_back, time_grouping=time_grouping)

if __name__ == "__main__":
    main()
